$ifndef _ILI9325_INC_
$define _ILI9325_INC_
'
' ------------------------------------------------------------------------------------------------------------
'| This source code is provided 'as is' and any express or implied warranties, including,                     |
'| but not limited to, the implied warranties of merchantability and fitness for a particular                 |
'| purpose are disclaimed.                                                                                    |
'|                                                                                                            |
'| In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,    |
'| special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute    |
'| goods or services; loss of use, data, or profits;  or business interruption) however caused and on any     |
'| theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)    |
'| arising in any way out of the use of this source code, even if advised of the possibility of such damage.  |
'|                                                                                                            |
'| Any views and conclusions contained in the source code and any documentation are those of the author's     |
'| and should not be interpreted as representing official policies,                                           |
'| either expressed or implied, of Crownhill Associates Ltd.                                                  |
' ------------------------------------------------------------------------------------------------------------
' 
' ILI9325 Colour Graphic LCD 8-bit interfacing library routines
' Written for PIC18F devices on the Proton Compiler by Les Johnson 24th December 2015
'
Declare Reminders = Off

    #disable Print                          ' Remove the compiler's Print library subroutine

    Declare LCD_Type = Graphic              ' Inform the compiler that we're using a graphic LCD
    Declare LCD_X_Res = 240
    Declare LCD_Y_Res = 320
    
$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif

'$define cGLCD_SingleDataPort 1             ' Uncomment if the LCD's data pins are connected to a single port on the microcontroller
'
' Setup the pins used by the LCD
'
$ifndef GLCD_RST_Pin
    $define GLCD_RST_Pin PORTA.4            ' Connects to the LCD's Reset pin
$endif

$ifndef GLCD_CS_Pin
    $define GLCD_CS_Pin PORTA.3             ' Connects to the LCD's CS pin
$endif

$ifndef GLCD_RS_Pin
    $define GLCD_RS_Pin PORTA.2             ' Connects to the LCD's RS pin
$endif

$ifndef GLCD_WR_Pin
    $define GLCD_WR_Pin PORTA.1             ' Connects to the LCD's WR pin
$endif

$ifndef GLCD_RD_Pin
    $define GLCD_RD_Pin PORTA.0             ' Connects to the LCD's RD pin
$endif
'
' Now the LCD's data pins
'
$ifdef cGLCD_SingleDataPort                 ' Are the LCD's D10 to D17 pins connected to a single port on the microcontroller?
    $ifndef GLCD_Data_Port
        $define GLCD_Data_Port PORTB        ' Yes. So define it (if required)
    $endif
$else                                       ' Otherwise... D10 to D17 are on various pins of various ports. So...
    $ifndef GLCD_D10_Pin
        $define GLCD_D10_Pin PORTB.7        ' Connects to the LCD's D10 pin
    $endif

    $ifndef GLCD_D11_Pin
        $define GLCD_D11_Pin PORTB.6        ' Connects to the LCD's D11 pin
    $endif

    $ifndef GLCD_D12_Pin
        $define GLCD_D12_Pin PORTC.5        ' Connects to the LCD's D12 pin
    $endif

    $ifndef GLCD_D13_Pin
        $define GLCD_D13_Pin PORTC.4        ' Connects to the LCD's D13 pin
    $endif

    $ifndef GLCD_D14_Pin
        $define GLCD_D14_Pin PORTC.3        ' Connects to the LCD's D14 pin
    $endif

    $ifndef GLCD_D15_Pin
        $define GLCD_D15_Pin PORTC.2        ' Connects to the LCD's D15 pin
    $endif

    $ifndef GLCD_D16_Pin
        $define GLCD_D16_Pin PORTC.1        ' Connects to the LCD's D16 pin
    $endif

    $ifndef GLCD_D17_Pin
        $define GLCD_D17_Pin PORTC.0        ' Connects to the LCD's D17 pin
    $endif
$endif 
   
$define cGLCD_Width 240
$define cGLCD_Height 320
$define cCGRAMSize 320 * 240
'
' ILI9325 Register names
'
$define cGLCD_START_OSC             $00
$define cGLCD_DRIV_OUT_CTRL         $01
$define cGLCD_DRIV_WAV_CTRL         $02
$define cGLCD_ENTRY_MOD             $03
$define cGLCD_RESIZE_CTRL           $04
$define cGLCD_DISP_CTRL1            $07
$define cGLCD_DISP_CTRL2            $08
$define cGLCD_DISP_CTRL3            $09
$define cGLCD_DISP_CTRL4            $0A
$define cGLCD_RGB_DISP_IF_CTRL1     $0C
$define cGLCD_FRM_MARKER_POS        $0D
$define cGLCD_RGB_DISP_IF_CTRL2     $0F
$define cGLCD_POW_CTRL1             $10
$define cGLCD_POW_CTRL2             $11
$define cGLCD_POW_CTRL3             $12
$define cGLCD_POW_CTRL4             $13
$define cGLCD_GRAM_HOR_AD           $20
$define cGLCD_GRAM_VER_AD           $21
$define cGLCD_RW_GRAM               $22
$define cGLCD_POW_CTRL7             $29
$define cGLCD_FRM_RATE_COL_CTRL     $2B
$define cGLCD_GAMMA_CTRL1           $30
$define cGLCD_GAMMA_CTRL2           $31
$define cGLCD_GAMMA_CTRL3           $32
$define cGLCD_GAMMA_CTRL4           $35
$define cGLCD_GAMMA_CTRL5           $36
$define cGLCD_GAMMA_CTRL6           $37
$define cGLCD_GAMMA_CTRL7           $38
$define cGLCD_GAMMA_CTRL8           $39
$define cGLCD_GAMMA_CTRL9           $3C
$define cGLCD_GAMMA_CTRL10          $3D
$define cGLCD_HOR_START_AD          $50
$define cGLCD_HOR_END_AD            $51
$define cGLCD_VER_START_AD          $52
$define cGLCD_VER_END_AD            $53
$define cGLCD_GATE_SCAN_CTRL1       $60
$define cGLCD_GATE_SCAN_CTRL2       $61
$define cGLCD_GATE_SCAN_CTRL3       $6A
$define cGLCD_PART_IMG1_DISP_POS    $80
$define cGLCD_PART_IMG1_START_AD    $81
$define cGLCD_PART_IMG1_END_AD      $82
$define cGLCD_PART_IMG2_DISP_POS    $83
$define cGLCD_PART_IMG2_START_AD    $84
$define cGLCD_PART_IMG2_END_AD      $85
$define cGLCD_PANEL_IF_CTRL1        $90
$define cGLCD_PANEL_IF_CTRL2        $92
$define cGLCD_PANEL_IF_CTRL3        $93
$define cGLCD_PANEL_IF_CTRL4        $95
$define cGLCD_PANEL_IF_CTRL5        $97
$define cGLCD_PANEL_IF_CTRL6        $98
'
' Pre-Defined colour codes
'                            RRRRRGGGGGGBBBBB
$define clBlack             %0000000000000000
$define clBrightBlue        %0000000000011111
$define clBrightGreen       %0000011111100000
$define clBrightCyan        %0000011111111111
$define clBrightRed         %1111100000000000
$define clBrightMagenta     %1111100000011111
$define clBrightYellow      %1111111111100000
$define clBlue              %0000000000010000
$define clGreen             %0000010000000000
$define clCyan              %0000010000010000
$define clRed               %1000000000000000
$define clMagenta           %1000000000010000
$define clBrown             %1111110000000000
$define clLightGray         %1000010000010000
$define clDarkGray          %0100001000001000
$define clLightBlue         %1000010000011111
$define clLightGreen        %1000011111110000
$define clLightCyan         %1000011111111111
$define clLightRed          %1111110000010000
$define clLightMagenta      %1111110000011111
$define clYellow            %1111111111110000
$define clWhite             %1111111111111111

$define clGray0             %1110011100011100
$define clGray1             %1100011000011000
$define clGray2             %1010010100010100
$define clGray3             %1000010000010000
$define clGray4             %0110001100001100
$define clGray5             %0100001000001000
$define clGray6             %0010000100000100
'
' Setup rotation constants
'
$define cGLCD_Portrait 0
$define cGLCD_Portrait_Reversed 1
$define cGLCD_Landscape 2
$define cGLCD_Landscape_Reversed 3
'
' Startup registers and data to place into them
'
    Dim ILI_wRegValues[86] As Code = As Word  cGLCD_START_OSC,        $0001,_
                                              cGLCD_DRIV_OUT_CTRL,    $0100,_
                                              cGLCD_DRIV_WAV_CTRL,    $0700,_
                                              cGLCD_ENTRY_MOD,        $1000,_
                                              cGLCD_RESIZE_CTRL,      $0000,_
                                              cGLCD_DISP_CTRL2,       $0202,_
                                              cGLCD_DISP_CTRL3,       $0000,_
                                              cGLCD_DISP_CTRL4,       $0000,_
                                              cGLCD_RGB_DISP_IF_CTRL1,$0000,_
                                              cGLCD_FRM_MARKER_POS,   $0000,_
                                              cGLCD_RGB_DISP_IF_CTRL2,$0000,_
                                              '
                                              ' Power On
                                              '
                                              cGLCD_POW_CTRL1,        $0000,_
                                              cGLCD_POW_CTRL2,        $0000,_
                                              cGLCD_POW_CTRL3,        $0000,_
                                              cGLCD_POW_CTRL4,        $0000,_
                                              cGLCD_POW_CTRL1,        $17B0,_
                                              cGLCD_POW_CTRL2,        $0037,_
                                              cGLCD_POW_CTRL3,        $0138,_
                                              cGLCD_POW_CTRL4,        $1700,_
                                              cGLCD_POW_CTRL7,        $000D,_
                                              cGLCD_GRAM_HOR_AD,      $0000,_
                                              cGLCD_GRAM_VER_AD,      $0000,_
                                              '
                                              ' Set Gamma
                                              '
                                              cGLCD_GAMMA_CTRL1,      $0001,_
                                              cGLCD_GAMMA_CTRL2,      $0606,_
                                              cGLCD_GAMMA_CTRL3,      $0304,_
                                              cGLCD_GAMMA_CTRL4,      $0103,_
                                              cGLCD_GAMMA_CTRL5,      $011D,_
                                              cGLCD_GAMMA_CTRL6,      $0404,_
                                              cGLCD_GAMMA_CTRL7,      $0404,_
                                              cGLCD_GAMMA_CTRL8,      $0404,_
                                              cGLCD_GAMMA_CTRL9,      $0700,_
                                              cGLCD_GAMMA_CTRL10,     $0A1F,_
                                              '
                                              ' Set GRAM area
                                              '
                                              cGLCD_HOR_START_AD,     $0000,_
                                              cGLCD_HOR_END_AD,       $00EF,_
                                              cGLCD_VER_START_AD,     $0000,_
                                              cGLCD_VER_END_AD,       $013F,_
                                              cGLCD_GATE_SCAN_CTRL1,  $2700,_
                                              cGLCD_GATE_SCAN_CTRL2,  $0001,_
                                              cGLCD_GATE_SCAN_CTRL3,  $0000,_
                                              '
                                              ' Panel display
                                              '
                                              cGLCD_PANEL_IF_CTRL1,   $0010,_
                                              cGLCD_PANEL_IF_CTRL2,   $0000,_
                                              cGLCD_PANEL_IF_CTRL4,   $0101,_
                                              '
                                              ' Panel Control
                                              '
                                              cGLCD_DISP_CTRL1,       $0173
'
' Create variables
'
    Dim tTransparent As Bit                     ' True if transparent text required. i.e. no paper colour
    
    Dim GLCD_wPixelXpos As Word
    Dim GLCD_wPixelYpos As Word
    
    Dim wPixelColour As Word System             ' The colour of the pixel or character
    Dim wPaperColour As Word System             ' The colour of the background (paper)
    Dim ILI_bRotation As Byte                   ' The current rotation value used
    Dim GLCD_wWidth As Word                     ' The maximum width of the LCD display
    Dim GLCD_wHeight As Word                    ' The maximum height of the LCD display
    
    Dim _GLCD_Xpos As Byte System               ' Bring the compiler's system variable into the program
    Dim _GLCD_XposH As Byte System              ' Bring the compiler's system variable into the program
    Dim GLCD_wCursorX As _GLCD_Xpos.Word        ' 16-bit Text cursor X position
    Dim GLCD_wCursorY As Word System            ' 16-bit Text cursor Y position

    Dim ILI_bIndex As Byte
$ifndef cGLCD_SingleDataPort                    ' Are the LCD's D10 to D17 pins connected to a single port on the microcontroller?    
    Dim ILI_bData As Byte System                ' No. So we need this variable created
$endif
    Dim ILI_wWriteData_Data As Word System
    Dim ILI_wWriteCommand_Cmd As Word System
    Dim ILI_wReadData_Data As Word
    Dim GLCD_wCls_Index As Dword  
    Dim ILI_wParam As GLCD_wCls_Index.Word
'
' GLCD_Circle and GLCD_Rectangle variables and aliases
'
    Dim GLCD_wQuadA As Word
    Dim GLCD_wQuadB As Word
    Dim GLCD_wQuadC As Word
    Dim GLCD_wQuadD As Word

    Dim GLCD_wXpos_S As Word
    Dim GLCD_wYpos_S As Word

    Dim GLCD_wSize As Word                      ' 16-bit Size of a circle, or a line, or a box
    Dim GLCD_wRectHeight As Word                ' 16-bit Height value
    Dim GLCD_wRectLength As Word                ' 16-bit Length value
   
    Dim GLCD_wTR As Word System
    Dim GLCD_wDD As Word System
    
    Dim GLCD_wXX As Word System
    Dim GLCD_wYY As Word System
'
' GLCD_Line variables aliased from GLCD_Circle variables
'
    Dim GLCD_wXpos_E As GLCD_wQuadA
    Dim GLCD_wYpos_E As GLCD_wQuadB
    Dim GLCD_wDeltaY As GLCD_wQuadC
    Dim GLCD_wDeltaX As GLCD_wQuadD

    Dim GLCD_bStepY As GLCD_wXX.Byte0
    Dim GLCD_wDeviation As GLCD_wTR
    Dim GLCD_bStepX As GLCD_wDD.Byte0
'
' Alias some SFRs as 16-bit variables
'
    Dim ILI_wTblPtr As TBLPTRL.Word
    Dim ILI_wProd As PRODL.Word
    Dim ILI_wFSR0 As FSR0L.Word
    Dim ILI_wFSR1 As FSR1L.Word
'
' Character display variables
'
    Dim GLCD_wHeaderAddress As Word System          ' Holds the address of the font offset table (in code memory)
    Dim GLCD_bFontAmountReads As Byte System        ' Holds the amount of bytes per Y segemnt of the character's font
    Dim GLCD_bFontHeight As Byte System             ' Holds the height of the character (in pixels)
    Dim GLCD_bFontWidth As Byte System              ' Holds the width of the font (in pixels) 
'
' Bring the system variables that the compiler's library uses for Print into the program
'
    Dim GPR As Byte System
    Dim GEN3 As Byte System
    Dim GEN3H As Byte System
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP3 As Byte System
    Dim PP4 As Byte System
    Dim PP3H As Byte System
    Dim PP5 As Byte System
    Dim PP5H As Byte System
    Dim PP6 As Byte System
    Dim PP6H As Byte System
'
' Alias variables used in GLCD_PrintChar to system variables
'
    Dim bSaveChar As PP3H
    Dim wSaveTblPtr As GEN3                         ' Used to save the contents of SFRs TBLPTRL\H
    Dim bCharReadsTemp As PP0
    Dim bCharPixelsTemp As PP0H
    Dim bHeightCount As PP3
    Dim bWidthCount As PP4
            
'----------------------------------------------------------------------------------
    GoTo _ILI9325_Main                              ' Jump over the subroutines
'----------------------------------------------------------------------------------
' Helper Macro that returns the current pixel colour         
' Input     : None                                                       
' Output    : Colour coded in format:                                    
'                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 
'                Colour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  
' Notes     : None                                                       
'
$define GLCD_GetPenColour() wPixelColour
$define GLCD_GetPenColor() wPixelColour

'----------------------------------------------------------------------------------
' Helper Macro that returns the current paper colour        
' Input     : None                                                       
' Output    : Colour coded in format:                                    
'                bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 
'             Colour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  
' Notes     : None                                                       
'
$define GLCD_GetPaperColour() wPaperColour
$define GLCD_GetPaperColor() wPaperColour

'----------------------------------------------------------------------------------
' Set current colour of a pixel                              
' Input     : wPixelColour coded in format:                              
'                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 
'                 Colour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  
' Output    : None                                                       
' Notes     : None                                                       
'
$define GLCD_PenColour(pColour) wPixelColour = pColour
$define GLCD_PenColor(pColour) wPixelColour = pColour

'----------------------------------------------------------------------------------
' Set current colour of the background                       
' Input     : wPaperColour coded in format:                         
'                   bits 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00 
'                Colour  R  R  R  R  R  G  G  G  G  G  G  B  B  B  B  B  
' Output    : None                                                       
' Notes     : None                                                       
'
$define GLCD_PaperColour(pColour) wPaperColour = pColour
$define GLCD_PaperColor(pColour) wPaperColour = pColour

'----------------------------------------------------------------------------------
' Make the pins connected to the LCD's D0 to D7 pins outputs
' Input     : None
' Output    : None
' Notes     : None
'
$ifdef cGLCD_SingleDataPort                         ' Are the LCD's Data pins connected to a single port on the microcontroller?

$define ILI9325_hSetWriteDir() Output GLCD_Data_Port

$else                                               ' Otherwise... The LCD's Data pins are connected to various pins of various ports. So...

$define ILI9325_hSetWriteDir() _ILI9325_hSetWriteDir

_ILI9325_hSetWriteDir Macro
    GoSub __ILI9325_SetWriteDir
Endm

#ifMacro- _ILI9325_hSetWriteDir
__ILI9325_SetWriteDir:
    Output GLCD_D10_Pin
    Output GLCD_D11_Pin
    Output GLCD_D12_Pin
    Output GLCD_D13_Pin
    Output GLCD_D14_Pin
    Output GLCD_D15_Pin
    Output GLCD_D16_Pin
    Output GLCD_D17_Pin
    Return
#endIfMacro-
$endif

'----------------------------------------------------------------------------------
' Make the pins connected to the LCD's D0 to D7 pins inputs
' Input     : None
' Output    : None
' Notes     : None
'
$ifdef cGLCD_SingleDataPort                         ' Are the LCD's Data pins connected to a single port on the microcontroller?

$define ILI9325_hSetReadDir() Input GLCD_Data_Port

$else                                               ' Otherwise... The LCD's Data pins are connected to various pins of various ports. So...

$define ILI9325_hSetReadDir() _ILI9325_hSetReadDir

_ILI9325_hSetReadDir Macro
    GoSub __ILI9325_SetReadDir
Endm

#ifMacro- _ILI9325_hSetReadDir
__ILI9325_SetReadDir:
    Input GLCD_D10_Pin
    Input GLCD_D11_Pin
    Input GLCD_D12_Pin
    Input GLCD_D13_Pin
    Input GLCD_D14_Pin
    Input GLCD_D15_Pin
    Input GLCD_D16_Pin
    Input GLCD_D17_Pin
    Return
#endIfMacro-
$endif

'----------------------------------------------------------------------------------
' Write to the LCD's data pins
' Input     : pData holds the data to write to the LCD
' Output    : None
' Notes     : None
'
$ifdef cGLCD_SingleDataPort                         ' Are the LCD's Data pins connected to a single port on the microcontroller?

$define ILI9325_hWrite8(pData) GLCD_Data_Port = pData

$else                                               ' Otherwise... The LCD's Data pins are connected to various pins of various ports. So...

$define ILI9325_hWrite8(pData) '
    ILI_bData = pData          '
    _ILI9325_hWrite8

_ILI9325_hWrite8 Macro
    GoSub __ILI9325_Write8
Endm

#ifMacro- _ILI9325_hWrite8
__ILI9325_Write8:
    Clear GLCD_D10_Pin
    If ILI_bData.0 = 1 Then
        Set GLCD_D10_Pin
    EndIf
    Clear GLCD_D11_Pin
    If ILI_bData.1 = 1 Then
        Set GLCD_D11_Pin
    EndIf
    Clear GLCD_D12_Pin
    If ILI_bData.2 = 1 Then
        Set GLCD_D12_Pin
    EndIf
    Clear GLCD_D13_Pin
    If ILI_bData.3 = 1 Then
        Set GLCD_D13_Pin
    EndIf
    Clear GLCD_D14_Pin
    If ILI_bData.4 = 1 Then
        Set GLCD_D14_Pin
    EndIf
    Clear GLCD_D15_Pin
    If ILI_bData.5 = 1 Then
        Set GLCD_D15_Pin
    EndIf
    Clear GLCD_D16_Pin
    If ILI_bData.6 = 1 Then
        Set GLCD_D16_Pin
    EndIf
    Clear GLCD_D17_Pin
    If ILI_bData.7 = 1 Then
        Set GLCD_D17_Pin
    EndIf
    Return
#endIfMacro-
$endif

'----------------------------------------------------------------------------------
' Read from the LCD's data pins
' Input     : None
' Output    : ILI_bData holds the data read from the LCD
' Notes     : None
'
$ifdef cGLCD_SingleDataPort                         ' Are the LCD's Data pins connected to a single port on the microcontroller?

$define ILI9325_hRead8(pResult) pResult = GLCD_Data_Port

$else                                               ' Otherwise... The LCD's Data pins are connected to various pins of various ports. So...

$define ILI9325_hRead8(pResult) '
    _ILI9325_hRead8             '
    pResult = ILI_bData

_ILI9325_hRead8 Macro
    GoSub __ILI9325_Read8
Endm

#ifMacro- _ILI9325_hRead8
__ILI9325_Read8:
    ILI_bData.0 = GLCD_D10_Pin
    ILI_bData.1 = GLCD_D11_Pin
    ILI_bData.2 = GLCD_D12_Pin
    ILI_bData.3 = GLCD_D13_Pin
    ILI_bData.4 = GLCD_D14_Pin
    ILI_bData.5 = GLCD_D15_Pin
    ILI_bData.6 = GLCD_D16_Pin
    ILI_bData.7 = GLCD_D17_Pin
    Return
#endIfMacro-
$endif

'----------------------------------------------------------------------------------
' Write data to the LCD
' Input     : pData holds the data to write to the LCD
' Output    : None
' Notes     : None
'
$define ILI9325_hWriteData(pData) '
    ILI_wWriteData_Data = pData   '
    _ILI9325_hWriteData

_ILI9325_hWriteData Macro
    GoSub __ILI9325_WriteData
Endm

#ifMacro- _ILI9325_hWriteData
__ILI9325_WriteData:
    Clear GLCD_CS_Pin
    Set GLCD_RS_Pin
    Set GLCD_RD_Pin
    Set GLCD_WR_Pin

    ILI9325_hSetWriteDir()                      ' Make the data pins outputs
    ILI9325_hWrite8(ILI_wWriteData_Data.Byte1)

    Clear GLCD_WR_Pin
    Set GLCD_WR_Pin

    ILI9325_hWrite8(ILI_wWriteData_Data.Byte0)

    Clear GLCD_WR_Pin
    Set GLCD_WR_Pin

    Set GLCD_CS_Pin
    Return
#endIfMacro-

'----------------------------------------------------------------------------------
' Write a command to the LCD
' Input     : pCmd holds the command to wtite to the register
' Output    : None
' Notes     : None
'
$define ILI9325_hWriteCommand(pCmd) '
    ILI_wWriteCommand_Cmd = pCmd    '
    _ILI9325_hWriteCommand

_ILI9325_hWriteCommand Macro
    GoSub __ILI9325_WriteCommand
Endm

#ifMacro- _ILI9325_hWriteCommand
__ILI9325_WriteCommand:
    Clear GLCD_CS_Pin
    Clear GLCD_RS_Pin
    Set GLCD_RD_Pin
    Set GLCD_WR_Pin

    ILI9325_hSetWriteDir()
    ILI9325_hWrite8(ILI_wWriteCommand_Cmd.Byte1)

    Clear GLCD_WR_Pin
    Set GLCD_WR_Pin

    ILI9325_hWrite8(ILI_wWriteCommand_Cmd.Byte0)

    Clear GLCD_WR_Pin
    Set GLCD_WR_Pin
    Set GLCD_CS_Pin
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------------
' Read data from the LCD
' Input     : pResult holds the data read from the register
' Output    : None
' Notes     : None
'
$define ILI9325_hReadData(pResult) '
    _ILI9325_hReadData             '
    pResult = ILI_wReadData_Data
    
_ILI9325_hReadData Macro
    GoSub __ILI9325_ReadData
Endm

#ifMacro- _ILI9325_hReadData
__ILI9325_ReadData:
    Clear GLCD_CS_Pin
    Set GLCD_RS_Pin
    Set GLCD_RD_Pin
    Set GLCD_WR_Pin

    ILI9325_hSetReadDir()

    Clear GLCD_RD_Pin
    DelayUS 10
    ILI9325_hRead8(ILI_wReadData_Data.Byte1)

    Set GLCD_RD_Pin
    Clear GLCD_RD_Pin

    DelayUS 10
    ILI9325_hRead8(ILI_wReadData_Data.Byte0)

    Set GLCD_RD_Pin
    Set GLCD_CS_Pin
    Return
#endIfMacro-

'----------------------------------------------------------------------------------
' Read one of the LCD's registers
' Input     : pAddress holds the address of the register
' Output    : pResult holds the data read from the register
' Notes     : None
'
$define ILI9325_hReadRegister(pAddress, pResult) '
   ILI9325_hWriteCommand(pAddress)               '
   ILI9325_hReadData(pResult)

'----------------------------------------------------------------------------------
' Write to one of the LCD's registers
' Input     : pAddress holds the address of the register
'           : Holds the data to write to the register
' Output    : None
' Notes     : None
'
$define ILI9325_hWriteRegister(pAddress, pData) '
    ILI9325_hWriteCommand(pAddress)             '
    ILI9325_hWriteData(pData)

'----------------------------------------------------------------------------------
' Move to the address within the LCD for a pixel
' Input     : pXpos holds the X position required
'           : pYpos holds the Y position required
'           : ILI_bRotation holds the rotation of the display (0, 1, 2, 3)
' Output    : None
' Notes     : None
'
$define ILI9325_hGoTo(pXpos, pYpos) '
    GLCD_wPixelXpos = pXpos         '
    GLCD_wPixelYpos = pYpos         '
    GoSub _ILI9325_hGoTo

_ILI9325_hGoTo:   
    If ILI_bRotation = cGLCD_Portrait Then                                  ' Is the display To be standard Portrait?
        ILI9325_hWriteRegister(cGLCD_GRAM_HOR_AD, GLCD_wPixelXpos)          ' Yes. So set the X position
        ILI9325_hWriteRegister(cGLCD_GRAM_VER_AD, GLCD_wPixelYpos)          ' Set the Y position
    ElseIf ILI_bRotation = cGLCD_Portrait_Reversed Then                     ' Is the display To be reversed Portrait?
        ILI9325_hWriteRegister(cGLCD_GRAM_HOR_AD, 239 - GLCD_wPixelXpos)    ' Yes. So set the X position
        ILI9325_hWriteRegister(cGLCD_GRAM_VER_AD, GLCD_wPixelYpos)          ' Set the Y position
    ElseIf ILI_bRotation = cGLCD_Landscape_Reversed Then                    ' Is the display to be reversed Landscape?
        ILI9325_hWriteRegister(cGLCD_GRAM_HOR_AD, 319 - GLCD_wPixelYpos)    ' Yes. So set the X position
        ILI9325_hWriteRegister(cGLCD_GRAM_VER_AD, GLCD_wPixelXpos)          ' Set the Y position
    ElseIf ILI_bRotation = cGLCD_Landscape Then                             ' Is the display To be standard Landscape?
        ILI9325_hWriteRegister(cGLCD_GRAM_HOR_AD, GLCD_wPixelYpos)          ' Yes. So set the X position
        ILI9325_hWriteRegister(cGLCD_GRAM_VER_AD, GLCD_wPixelXpos)          ' Set the Y position
    EndIf
    ILI9325_hWriteCommand(cGLCD_RW_GRAM)
    Return

'----------------------------------------------------------------------------------
' Reset the LCD
' Input     : None
' Output    : None
' Notes     : None
'
$define ILI9325_hReset() GoSub _ILI9325_Reset

_ILI9325_Reset:
    Clear GLCD_RST_Pin
    DelayMS 100
    Set GLCD_RST_Pin
    Return

'----------------------------------------------------------------------------------
' Get the current rotation value
' Input     : None
' Output    : ILI_bRotation holds the current rotation value
' Notes     : None
'
$define GLCD_GetRotation() ILI_bRotation

'----------------------------------------------------------------------------------
' Alter the rotation of the LCD's display
' Input     : pRotation holds the rotation required 0, 1, 2, or 3
' Output    : None
' Notes     : None
'
$define GLCD_SetRotation(pRotation) '
    ILI_bRotation = pRotation       '
    _GLCD_SetRotation

_GLCD_SetRotation Macro-
    GoSub __GLCD_SetRotation
Endm

#ifMacro- _GLCD_SetRotation                                     ' Use this block of code if the macro is used in the program
__GLCD_SetRotation:
    ILI_bRotation = ILI_bRotation & %00000011                   ' Make sure the value does not go over 3
    If ILI_bRotation = cGLCD_Portrait Then                      ' Do we require standard Portrait?
        ILI9325_hWriteRegister(cGLCD_DRIV_OUT_CTRL, $0100)      ' Yes. So set Driver Output Control (Register $01)
        ILI9325_hWriteRegister(cGLCD_ENTRY_MOD, $1030)
        ILI9325_hWriteRegister(cGLCD_GATE_SCAN_CTRL1, $A700)    ' Gate Scan Line (Register $60)
        GLCD_wWidth = cGLCD_Width                               ' Adjust the width for the routines
        GLCD_wHeight = cGLCD_Height                             ' Adjust the height for the routines
        
    ElseIf ILI_bRotation = cGLCD_Portrait_Reversed Then         ' Do we require reversed Portrait? 
        ILI9325_hWriteRegister(cGLCD_DRIV_OUT_CTRL, $0100)      ' Yes. So set Driver Output Control (Register $01)
        ILI9325_hWriteRegister(cGLCD_ENTRY_MOD, $1038)
        ILI9325_hWriteRegister(cGLCD_GATE_SCAN_CTRL1, $2700)    ' Gate Scan Line (Register $60)
        GLCD_wWidth = cGLCD_Width                               ' Adjust the width for the routines
        GLCD_wHeight = cGLCD_Height                             ' Adjust the height for the routines
        
    ElseIf ILI_bRotation = cGLCD_Landscape_Reversed Then        ' Do we require standard Landscape? 
        ILI9325_hWriteRegister(cGLCD_DRIV_OUT_CTRL, $0000)      ' Yes. So set Driver Output Control (Register $01)
        ILI9325_hWriteRegister(cGLCD_ENTRY_MOD, $1030)
        ILI9325_hWriteRegister(cGLCD_GATE_SCAN_CTRL1, $2700)    ' Gate Scan Line (Register $60)
        GLCD_wWidth = cGLCD_Height                              ' Adjust the width for the routines
        GLCD_wHeight = cGLCD_Width                              ' Adjust the height for the routines
        
    ElseIf ILI_bRotation = cGLCD_Landscape Then                 ' Do we require reversed Landscape? 
        ILI9325_hWriteRegister(cGLCD_DRIV_OUT_CTRL, $0000)      ' Yes. So set Driver Output Control (Register $01)
        ILI9325_hWriteRegister(cGLCD_ENTRY_MOD, $1038)
        ILI9325_hWriteRegister(cGLCD_GATE_SCAN_CTRL1, $A700)    ' Gate Scan Line (Register $60)
        GLCD_wWidth = cGLCD_Height                              ' Adjust the width for the routines
        GLCD_wHeight = cGLCD_Width                              ' Adjust the height for the routines
    EndIf
    Return
#endIfMacro-

'----------------------------------------------------------------------------------
' Initialise the ILI9325 display chip
' Input     : None
' Output    : None
' Notes     : None
'
$define ILI9325_hInitDisplay() GoSub _ILI9325_hInitDisplay

_ILI9325_hInitDisplay:
    High GLCD_CS_Pin                                                ' \
    High GLCD_RS_Pin                                                ' |
    High GLCD_WR_Pin                                                ' | Make the pins outputs and disable the LCD
    High GLCD_RD_Pin                                                ' |
    High GLCD_RST_Pin                                               ' /

    ILI9325_hReset()                                                ' Reset the LCD
'
' Initialise the LCD
'
    ILI_bIndex = 0
    Repeat                                                          ' Create a loop to read the code memory table
        ILI9325_hWriteCommand(CRead16 ILI_wRegValues[ILI_bIndex])   ' Read and send the command value to the LCD
        Inc ILI_bIndex                                              ' Increment the index value to the data in the table
        ILI9325_hWriteData(CRead16 ILI_wRegValues[ILI_bIndex])      ' Read and send the data value to the LCD
        DelayMS 5
        Inc ILI_bIndex                                              ' Increment the index value to the next command in the table
    Until ILI_bIndex > 86                                           ' Until all the code memory table has been read
    
    GLCD_SetRotation(cGLCD_Landscape)                               ' Set the rotation to landscape
    Return
   
'----------------------------------------------------------------------------------
' Clear the LCD's display in a chosen colour
' Input     : wPaperColour holds the RGB565 colour to fill the LCD's screen
' Output    : None
' Notes     : None
'
$define GLCD_Cls(pColour)  '
    wPaperColour = pColour '
    _GLCD_Cls

_GLCD_Cls Macro-
    GoSub __GLCD_Cls
Endm

#ifMacro- _GLCD_Cls                                                 ' Use this block of code if the macro is used in the program
__GLCD_Cls:
    ILI9325_hGoTo(0,0)

    Clear GLCD_CS_Pin
    Set GLCD_RS_Pin
    Set GLCD_RD_Pin
    Set GLCD_WR_Pin

    ILI9325_hSetWriteDir()
    GLCD_wCls_Index = cCGRAMSize
    Repeat
        ILI9325_hWrite8(wPaperColour.Byte1)
        Clear GLCD_WR_Pin
        Set GLCD_WR_Pin
        ILI9325_hWrite8(wPaperColour.Byte0)
        Clear GLCD_WR_Pin
        Set GLCD_WR_Pin    
        Dec GLCD_wCls_Index
    Until GLCD_wCls_Index = 0

    Set GLCD_CS_Pin
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Overview  : Place a pixel on the LCD
' Input     : GLCD_wPixelXpos holds the pixel's X position
'           : GLCD_wPixelYpos holds the pixel's Y position
'           : wPixelColour holds the pixel's colour
' Output    : None
' Notes     : None
'
$define GLCD_Plot(pXpos, pYpos) '
    GLCD_wPixelYpos = pYpos     '
    GLCD_wPixelXpos = pXpos     '
    _GLCD_Plot

_GLCD_Plot Macro-
    GoSub __GLCD_Plot
Endm

#ifMacro- _GLCD_Plot, _GLCD_PrintChar, _GLCD_Print, _GLCD_Vline, _GLCD_Hline, _GLCD_Line, _GLCD_Circle
__GLCD_Plot:
'
' Check boundaries
'
    If GLCD_wPixelXpos.15 = 1 Then Return                   ' Exit if the Xpos is less than 0
    If GLCD_wPixelYpos.15 = 1 Then Return                   ' Exit if the Ypos is less than 0
    If GLCD_wPixelYpos >= GLCD_wHeight Then Return          ' Exit if the Ypos is greater than the LCD's Y resolution
    If GLCD_wPixelXpos >= GLCD_wWidth Then Return           ' Exit if the Xpos is greater than the LCD's X resolution

    ILI9325_hGoTo(GLCD_wPixelXpos, GLCD_wPixelYpos)         ' Point to the correct location in GRAM
    Clear GLCD_CS_Pin                                       ' Enable the LCD's interface    
    ILI9325_hWriteData(wPixelColour)                        ' Write the pixel to the LCD
    Set GLCD_CS_Pin                                         ' Disable the LCD's interface
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a Circle on the Graphic LCD
' Input     : GLCD_wXpos_S holds the starting X position
'           : GLCD_wYpos_S holds the starting Y position
'           : GLCD_wSize holds the radius of the circle
' Output    : None
' Notes     : None
'
$define GLCD_Circle(pXpos, pYpos, pRadius) '
    GLCD_wYpos_S = pYpos                   '
    GLCD_wSize = pRadius                   '
    GLCD_wXpos_S = pXpos                   '
    _GLCD_Circle

_GLCD_Circle Macro-
    GoSub __GLCD_Circle
Endm

#ifMacro- _GLCD_Circle                                      ' Add this block only if the macro has been used in the program
__GLCD_Circle:
    If GLCD_wSize = 0 Then Return                           ' Trap a radius of 0
    GLCD_wDD = GLCD_wYpos_S - GLCD_wXpos_S
    GLCD_wXX = 0
    GLCD_wYY = GLCD_wSize
    GLCD_wTR = 3 - (2 * GLCD_wSize)
    While GLCD_wXX <= GLCD_wYY
        GLCD_wQuadA = GLCD_wXpos_S + GLCD_wXX
        GLCD_wQuadB = GLCD_wYpos_S + GLCD_wYY
        GLCD_wQuadC = GLCD_wXpos_S - GLCD_wXX
        GLCD_wQuadD = GLCD_wYpos_S - GLCD_wYY
        GoSub _GLCD_PlotIt
        GLCD_wQuadA = GLCD_wYpos_S + GLCD_wYY - GLCD_wDD
        GLCD_wQuadB = GLCD_wYpos_S + GLCD_wXX
        GLCD_wQuadC = GLCD_wYpos_S - GLCD_wYY - GLCD_wDD
        GLCD_wQuadD = GLCD_wYpos_S - GLCD_wXX
        GoSub _GLCD_PlotIt
        If GLCD_wTR.15 = 1 Then
            GLCD_wTR = GLCD_wTR + 6
            GLCD_wTR = GLCD_wTR + (4 * GLCD_wXX)
        Else
            GLCD_wTR = GLCD_wTR + 10
            GLCD_wTR = GLCD_wTR + (4 * (GLCD_wXX - GLCD_wYY))
            Dec GLCD_wYY
        EndIf
        Inc GLCD_wXX
    Wend
    Return
'
' Plot 4 locations simultaneously
' This is part of the GLCD_Circle subroutine
'
_GLCD_PlotIt:
    GLCD_Plot(GLCD_wQuadA, GLCD_wQuadB)
    GLCD_Plot(GLCD_wQuadA, GLCD_wQuadD)
    GLCD_Plot(GLCD_wQuadC, GLCD_wQuadB)
    GLCD_Plot(GLCD_wQuadC, GLCD_wQuadD)
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a line from GLCD_wXpos_S, GLCD_wYpos_S to GLCD_wXpos_E, GLCD_wYpos_E
' Input     : GLCD_wXpos_S = Starting X position of the Line
'           : GLCD_wYpos_S = Starting Y position of the Line
'           : GLCD_wXpos_E = Ending X position of the Line
'           : GLCD_wYpos_E = Ending Y position of the Line
' Output    : None
' Notes     :
'
$define GLCD_Line(pStartXpos, pStartYpos, pEndXpos, pEndYpos) '
    GLCD_wYpos_S = pStartYpos                                 '
    GLCD_wXpos_E = pEndXpos                                   '
    GLCD_wYpos_E = pEndYpos                                   '
    GLCD_wXpos_S = pStartXpos                                 '
    _GLCD_Line

_GLCD_Line Macro-
    GoSub __GLCD_Line
Endm

#ifMacro- _GLCD_Line, _GLCD_LineTo, _GLCD_Rectangle                ' Add this block only if the macros have been used in the program
__GLCD_Line:
    GLCD_wDeltaY = GLCD_wYpos_E - GLCD_wYpos_S                      ' Find the initial Y direction
    GLCD_wDeltaX = GLCD_wXpos_E - GLCD_wXpos_S                      ' Find the initial X direction

    GLCD_bStepY.SByte = 1                                           ' Default to a positive step for Ypos
    If GLCD_wDeltaY.15 = 1 Then                                     ' Check if negative
        GLCD_wDeltaY = -GLCD_wDeltaY                                ' Negate GLCD_wDeltaY
        GLCD_bStepY.SByte = -1                                      ' And indicate a negative step
    EndIf
    GLCD_bStepX.SByte = 1                                           ' Default to a positive step for Xpos
    If GLCD_wDeltaX.15 = 1 Then                                     ' Check if negative
        GLCD_wDeltaX = -GLCD_wDeltaX                                ' Negate GLCD_wDeltaX
        GLCD_bStepX.SByte = -1                                      ' And indicate a negative step
    EndIf
    GLCD_Plot(GLCD_wXpos_S,GLCD_wYpos_S)                            ' The first pixel plot is a special case

    If GLCD_wDeltaX > GLCD_wDeltaY Then
        GLCD_wDeviation.SWord = GLCD_wDeltaY - GLCD_wDeltaX
        While GLCD_wXpos_S <> GLCD_wXpos_E
            If GLCD_wDeviation.15 = 0 Then                          ' Check if positive
                GLCD_wYpos_S = GLCD_wYpos_S + GLCD_bStepY.SByte     ' Move on the Y axis directed by GLCD_bStepY
                GLCD_wDeviation.SWord = GLCD_wDeviation.SWord - GLCD_wDeltaX
            EndIf
            GLCD_wXpos_S = GLCD_wXpos_S + GLCD_bStepX.SByte         ' Move on the X axis directed by GLCD_bStepX
            GLCD_wDeviation.SWord = GLCD_wDeviation.SWord + GLCD_wDeltaY
            GLCD_Plot(GLCD_wXpos_S,GLCD_wYpos_S)                    ' Draw or Erase the pixel at this position
        Wend
        Return                                                      ' Return prematurely (this saves an else command)
    EndIf
    GLCD_wDeviation.SWord = GLCD_wDeltaX - GLCD_wDeltaY
    
    While GLCD_wYpos_S <> GLCD_wYpos_E
        If GLCD_wDeviation.15 = 0 Then                              ' Check if positive
            GLCD_wXpos_S = GLCD_wXpos_S + GLCD_bStepX.SByte         ' Move on the X axis directed by GLCD_bStepX
            GLCD_wDeviation.SWord = GLCD_wDeviation.SWord - GLCD_wDeltaY
        EndIf
        GLCD_wYpos_S = GLCD_wYpos_S + GLCD_bStepY.SByte             ' Move on the Y axis directed by GLCD_bStepY
        GLCD_wDeviation.SWord = GLCD_wDeviation.SWord + GLCD_wDeltaX
        GLCD_Plot(GLCD_wXpos_S,GLCD_wYpos_S)                        ' Draw or Erase the pixel at this position
    Wend    
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a line from the previous end X and Y positions to new X and Y coordinates
' Input     : GLCD_wXpos_E holds the X position end
'           : ILI_wProd holds the Y position end
' Output    : None
' Notes     : None
'
$define GLCD_LineTo(pEndXpos, pEndYpos) '
    ILI_wParam = pEndXpos               '
    ILI_wProd = pEndYpos                '  
    _GLCD_LineTo

_GLCD_LineTo Macro-
    GoSub __GLCD_LineTo
Endm

#ifMacro- _GLCD_LineTo                              ' Add this block only if the macro has been used in the program
__GLCD_LineTo:
    GLCD_wXpos_S = GLCD_wXpos_E                     ' Transfer previous X pos end to X pos start
    GLCD_wYpos_S = GLCD_wYpos_E                     ' Transfer the previous Y pos to the Y pos start
    GLCD_wXpos_E = ILI_wParam                       ' Transfer the new end X position
    GLCD_wYpos_E = ILI_wProd                        ' Transfer the new end Y position
    GoSub __GLCD_Line                               ' Draw a line
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a vertical line
' Input     : GLCD_wPixelXpos = Starting X position of the Line
'           : GLCD_wPixelYpos = Starting Y position of the Line
'           : GLCD_wSize holds the length of the line
' Output    : None
' Notes     : None
'
$define GLCD_Vline(pXpos, pYpos, pSize) '
    GLCD_wPixelYpos = pYpos             '
    GLCD_wSize = pSize                  '
    GLCD_wPixelXpos = pXpos             '
    _GLCD_Vline

_GLCD_Vline Macro-
    GoSub __GLCD_Vline
Endm

#ifMacro- _GLCD_Vline                               ' Add this block only if the macro has been used in the program
__GLCD_Vline:
    Repeat
        GLCD_Plot(GLCD_wPixelXpos, GLCD_wPixelYpos)
        Inc GLCD_wPixelYpos
    Until GLCD_wPixelYpos > GLCD_wSize
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a horizontal line
' Input     : GLCD_wPixelXpos = Starting X position of the Line
'           : GLCD_wPixelYpos = Starting Y position of the Line
'           : GLCD_wSize holds the length of the line
' Output    : None
' Notes     : None
'
$define GLCD_Hline(pXpos, pYpos, pSize) '
    GLCD_wPixelYpos = pYpos             '
    GLCD_wSize = pSize                  '
    GLCD_wPixelXpos = pXpos             '
    _GLCD_Hline

_GLCD_Hline Macro-
    GoSub __GLCD_Hline
Endm

#ifMacro- _GLCD_Hline                               ' Add this block only if the macro has been used in the program
__GLCD_Hline:
    Repeat
        GLCD_Plot(GLCD_wPixelXpos, GLCD_wPixelYpos)
        Inc GLCD_wPixelXpos
    Until GLCD_wPixelXpos > GLCD_wSize
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a Rectangle
' Input     : GLCD_wXpos_S holds the starting X position
'           : GLCD_wYpos_S holds the starting Y position
'           : GLCD_wRectLength holds the length of the rectangle
'           : GLCD_wRectHeight holds the height of the rectangle
' Output    : None
' Notes     :
'
$define GLCD_Rectangle(pXpos, pYpos, pLength, pHeight) '
    GLCD_wYpos_S = pYpos                               '
    GLCD_wRectLength = pLength                         '
    GLCD_wRectHeight = pHeight                         '
    GLCD_wXpos_S = pXpos                               '
    _GLCD_Rectangle

_GLCD_Rectangle Macro-
    GoSub __GLCD_Rectangle
Endm

#ifMacro- _GLCD_Rectangle                           ' Add this block only if the macro has been used in the program
__GLCD_Rectangle:
    GLCD_Line(GLCD_wXpos_S, GLCD_wYpos_S, GLCD_wXpos_S + GLCD_wRectLength, GLCD_wYpos_S)  ' Draw top line

    GLCD_wYpos_S = GLCD_wYpos_E                     ' Transfer the previous Y pos end to Y pos start
    GLCD_wXpos_S = GLCD_wXpos_E                     ' Transfer the previous X pos end to X pos start
    GLCD_wYpos_E = GLCD_wYpos_E + GLCD_wRectHeight
    GoSub __GLCD_Line                               ' Draw right line
    
    GLCD_wYpos_S = GLCD_wYpos_E                     ' Transfer the previous Y pos end to Y pos start
    GLCD_wXpos_S = GLCD_wXpos_E                     ' Transfer the previous X pos end to X pos start
    GLCD_wXpos_E = GLCD_wXpos_E - GLCD_wRectLength  
    GoSub __GLCD_Line                               ' Draw bottom line

    GLCD_wYpos_S = GLCD_wYpos_E                     ' Transfer the previous Y pos end to Y pos start
    GLCD_wXpos_S = GLCD_wXpos_E                     ' Transfer the previous X pos end to X pos start
    GLCD_wYpos_E = GLCD_wYpos_E - GLCD_wRectHeight
    GoSub __GLCD_Line                               ' Draw left line
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Draw a triangle
' Input     : pStartXpos holds the starting X position
'           : pStartYpos holds the starting Y position
'           : pApexXpos holds the apex X position
'           : pApexYpos holds the apex Y position
'           : pStartXpos holds the endinf X position
'           : pStartYpos holds the endinf Y position
' Output    : None
' Notes     : None
'
$define GLCD_Triangle(pStartXpos, pStartYpos, pApexXpos, pApexYpos, pEndXpos, pEndYpos) '
    GLCD_Line(pStartXpos, pStartYpos, pApexXpos, pApexYpos)                             '
    GLCD_LineTo(pEndXpos, pEndYpos)                                                     '
    GLCD_LineTo(pStartXpos, pStartYpos)

'-------------------------------------------------------------------------------------------------
' Move the cursor values (in pixels)
' Input     : GLCD_wCursorY = 16-bit Y position
'           : WREG = 8-bit X position
' Output    : None
' Notes     : None
'
$define GLCD_Cursor(pXpos, pYpos) '
    GLCD_wCursorY = pYpos         '
    GLCD_wCursorX = pXpos

'-------------------------------------------------------------------------------------------------
' Choose a font for character display
' Input     : TBLPTRL\H hold the font offset table address
' Output    : GLCD_wHeaderAddress holds the start of the font offset table address
'           : GLCD_bFontHeight holds the height of the font (in pixels)
'           : GLCD_bFontAmountReads holds the amount of bytes per font segment
' Notes     : None
'
$define GLCD_SetFont(pFont)        '
    ILI_wTblPtr = AddressOf(pFont) '
    _SetFont

_SetFont Macro-
    GoSub __SetFont
    Endm

#ifMacro- _SetFont
__SetFont:
    Tblrd*+                                             ' \ Read and store the font height from the font table
    GLCD_bFontHeight = TABLAT                           ' /
    Tblrd*+                                             ' Dummy read to increment the code memory address
    GLCD_wHeaderAddress = ILI_wTblPtr                   ' Store the current location of the font offset table
'
' Find the amount of bytes per font segment
'
    GLCD_bFontAmountReads = GLCD_bFontHeight / 8        ' Find the amount of bytes
    PRODL = GLCD_bFontHeight // 8                       ' Find the remainder
    If PRODL <> 0 Then                                  ' Is there a remainder?
        Inc GLCD_bFontAmountReads                       ' Yes. So increment the amount of bytes
    EndIf
    Return
#endIfMacro-

'-------------------------------------------------------------------------------------------------
' Display an ASCII character
' Input     : WREG holds the character to display
'           : If tTransparent = true then do not use a paper colour
' Output    : WREG still contains the character that was displayed
' Notes     : GLCD_wCursorY holds the Y position (in pixels)
'           : GLCD_wCursorX holds the X position (in pixels)
'
$define GLCD_PrintChar(pChar) '
    WREG = pChar              '
    _GLCD_PrintChar

_GLCD_PrintChar Macro-
    GoSub __GLCD_PrintChar
    Endm

#ifdef Print#Req
@Print
__GLCD_PrintChar:
    wSaveTblPtr = ILI_wTblPtr                                   ' Save the contents of TBLPTRL\H
'
' Locate the character's image data from code memory tables
'
    bSaveChar = WREG                                            ' Save the character value
    WREG = WREG - 32                                            ' Remove the ASCII offset
    WREG = WREG * 2                                             ' Offsets are 2 bytes each
    ILI_wTblPtr = GLCD_wHeaderAddress + WREG                    ' Point to the font image offset within the the table
    Tblrd*+                                                     ' \ Read and store the low byte of the image address
    ILI_wProd.Byte0 = TABLAT                                    ' /
    Tblrd*                                                      ' \ Read and store the high byte of the image address
    ILI_wProd.Byte1 = TABLAT                                    ' /
    ILI_wTblPtr = ILI_wProd                                     ' Point to the image address of the font
    Tblrd*+                                                     ' \ Read the font width value then increment to skip over it
    GLCD_bFontWidth = TABLAT                                    ' /
'
' Place the character on the LCD
'
    GLCD_wPixelXpos = GLCD_wCursorX                             ' Place the current cursor X position into the pixel X position
    bWidthCount = GLCD_bFontWidth                               ' Place the font width value into its working variable
_FontWidthLoop:
        GLCD_wPixelYpos = GLCD_wCursorY                         ' Place the current cursor Y position into the pixel Y position
        bHeightCount = 0                                        ' Clear the character height counter
        bCharReadsTemp = GLCD_bFontAmountReads                  ' Store the amount of bytes to read for this sgment
_FontBytesLoop:
        Tblrd*+                                                 ' Read the byte from code memory
        bCharPixelsTemp = 8                                     ' Create a loop for the bits to plot
_CharByteLoop:
            If tTransparent = False Then                        ' Do we require transparent background?
                ILI_wProd = wPixelColour                        ' No. So save the colour of the pixel
                If TABLAT.0 = 0 Then                            ' Do we need to set a pixel?
                    wPixelColour = wPaperColour                 ' No. So use the paper (background) colour
                EndIf
                GLCD_Plot(GLCD_wPixelXpos, GLCD_wPixelYpos)     ' Place a pixel on the LCD
                wPixelColour = ILI_wProd                        ' Restore the colour of the pixel
            Else                                                ' Otherwise...
                If TABLAT.0 = 1 Then                            ' Do we need to set a pixel?
                    GLCD_Plot(GLCD_wPixelXpos, GLCD_wPixelYpos) ' Yes. So place a pixel on the LCD
                EndIf
            EndIf
            Ror TABLAT                                          ' Move each pixel into bit-0
            Inc GLCD_wPixelYpos                                 ' Move down the LCD for the next pixel
            Inc bHeightCount                                    ' Increment the character font height counter
            If bHeightCount >= GLCD_bFontHeight Then _ExitLoop  ' Exit the loop when we reach the correct size of the character
            Djnz bCharPixelsTemp,_CharByteLoop                  ' Close the loop when all pixels plotted
        Djnz bCharReadsTemp,_FontBytesLoop                      ' Close the loop when all bytes read
_ExitLoop:
        Inc GLCD_wPixelXpos                                     ' Next X position
    Djnz bWidthCount,_FontWidthLoop                             ' Close the loop when all width pixels are done
    GLCD_wCursorX = GLCD_wPixelXpos                             ' Update the cursor position ready for the next character
    ILI_wTblPtr = wSaveTblPtr                                   ' Restore the contents of TBLPTRL\H
    WREG = bSaveChar                                            ' Restore the original character in WREG before returning
    Return
#endif

'-------------------------------------------------------------------------------------------------
' Display text
' Input     : Uses a single modifier from the compiler's Print routine
' Output    : None
' Notes     : This routine hi-jacks the compiler's Print library
'
$define GLCD_Print(pStringName) '
    _GLCD_Print                 '
    Print pStringName

_GLCD_Print Macro pStringName
Endm
    
'----------------------------------------------------------------------------------------------------
'   
_ILI9325_Main:
    ILI9325_hInitDisplay()                                      ' Initialise the display

$endif    

